import magic
from django.core.exceptions import ValidationError
from django.utils.deconstruct import deconstructible
from django.utils.translation import gettext_lazy as _


@deconstructible
class MimeTypeValidator:
    """
    mimetype are defined in /usr/share/file/magic.mgc
    """

    message = _(
        "File mimetype “%(mimetype)s” is not allowed. "
        "Allowed mimetypes are: %(allowed_mimetypes)s."
    )
    code = "invalid_mimetype"

    def __init__(self, allowed_mimetypes=None, message=None, code=None):
        if allowed_mimetypes is not None:
            allowed_mimetypes = [
                allowed_mimetypes.lower() for allowed_mimetypes in allowed_mimetypes
            ]
        self.allowed_mimetypes = allowed_mimetypes
        if message is not None:
            self.message = message
        if code is not None:
            self.code = code

    def __call__(self, value):
        mimetype = magic.from_buffer(value.read(4096), mime=True)
        value.seek(0)
        if (
            self.allowed_mimetypes is not None
            and mimetype not in self.allowed_mimetypes
        ):
            raise ValidationError(
                self.message,
                code=self.code,
                params={
                    "mimetype": mimetype,
                    "allowed_mimetypes": ", ".join(self.allowed_mimetypes),
                },
            )

    def __eq__(self, other):
        return (
            isinstance(other, self.__class__)
            and self.allowed_mimetypes == other.allowed_mimetypes
            and self.message == other.message
            and self.code == other.code
        )



from datetime import datetime

from rest_framework import serializers
from rest_framework.fields import ListField


class StringArrayField(ListField):
    def to_representation(self, obj):
        return ",".join(str(element) for element in obj.all())

    def to_internal_value(self, data):
        return super().to_internal_value(data.split(","))


class NestedSerializerWriteableRelationMixin:
    def __init__(self, **kwargs):
        self.serializer = kwargs.pop("serializer", None)
        if self.serializer is not None and not issubclass(
            self.serializer, serializers.Serializer
        ):
            raise TypeError('"serializer" is not a valid serializer class')
        super().__init__(**kwargs)

    def use_pk_only_optimization(self):
        return False if self.serializer else True

    def to_representation(self, instance):
        if self.serializer:
            return self.serializer(instance, context=self.context).data
        return super().to_representation(instance)


class TimestampField(serializers.DateTimeField):
    """
    Convert a django datetime to/from timestamp.
    """

    def to_representation(self, value):
        """
        Convert the field to its internal representation (aka timestamp)
        :param value: the DateTime value
        :return: a UTC timestamp integer
        """
        return value.timestamp()

    def to_internal_value(self, value):
        """
        deserialize a timestamp to a DateTime value
        :param value: the timestamp value
        :return: a django DateTime value
        """
        return datetime.fromtimestamp(float(value))



from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin

from .forms import CustomUserChangeForm, CustomUserCreationForm


class EmailUserAdmin(UserAdmin):
    add_form = CustomUserCreationForm
    form = CustomUserChangeForm
    model = get_user_model()
    list_display = (
        "email",
        "is_superuser",
        "is_staff",
        "is_active",
    )
    list_filter = (
        "email",
        "is_superuser",
        "is_staff",
        "is_active",
    )
    fieldsets = (
        (None, {"fields": ("email", "password")}),
        ("Permissions", {"fields": ("is_superuser", "is_staff", "is_active")}),
    )
    add_fieldsets = (
        (
            None,
            {
                "classes": ("wide",),
                "fields": (
                    "email",
                    "password1",
                    "password2",
                    "is_superuser",
                    "is_staff",
                    "is_active",
                ),
            },
        ),
    )
    search_fields = ("email",)
    ordering = ("email",)



# Generated by Django 3.1.8 on 2021-05-18 22:29

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Request',
            fields=[
                ('id', models.BigAutoField(editable=False, primary_key=True, serialize=False)),
                ('gid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('user', models.UUIDField()),
                ('method', models.CharField(max_length=8)),
                ('status', models.IntegerField()),
                ('ip', models.CharField(max_length=16)),
                ('request_data', models.CharField(max_length=8192)),
                ('response_data', models.CharField(max_length=8192)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 3.1.8 on 2021-06-09 22:31

from django.db import migrations, models
import django.utils.timezone
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('id', models.BigAutoField(editable=False, primary_key=True, serialize=False)),
                ('gid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('email', models.EmailField(max_length=254, unique=True, verbose_name='email address')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'abstract': False,
            },
        ),
    ]



import geoip2
from django.contrib.auth import get_user_model
from django.contrib.gis.geoip2 import GeoIP2
from django.http import HttpRequest


def ip_to_country(ip):
    g = GeoIP2()
    try:
        return g.country(ip)["country_code"]
    except geoip2.errors.AddressNotFoundError:
        return None


class AuthenticatedHttpRequest(HttpRequest):
    user: get_user_model()


def request_to_ip(request):
    x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
    if x_forwarded_for:
        return x_forwarded_for.split(",")[0]
    return request.META.get("REMOTE_ADDR")



import datetime
import os
import sys

import environ
from configurations import Configuration, values

env = environ.Env()


def to_bool(value):
    return value.lower() in ["true", "yes", "y", "t", "1", "on"]


class BaseCommon(Configuration):
    p = os.environ["DJANGO_SETTINGS_MODULE"].split(".")[0]
    TESTING = "test" in sys.argv
    ROOT_URLCONF = f"{p}.base.urls"
    WSGI_APPLICATION = f"{p}.base.wsgi.application"

    TOKEN_EXPIRE_SECONDS = int(os.environ.setdefault("TOKEN_EXPIRE_SECONDS", "0"))

    EMAIL_HOST = os.environ.setdefault("EMAIL_HOST", "unset_email_host")
    EMAIL_HOST_USER = os.environ.setdefault("EMAIL_HOST_USER", "unset_email_host_user")
    EMAIL_HOST_PASSWORD = os.environ.setdefault(
        "EMAIL_HOST_PASSWORD", "unset_email_host_password"
    )
    EMAIL_PORT = os.environ.setdefault("EMAIL_PORT", "unset_email_port")
    EMAIL_USE_TLS = to_bool(os.environ.setdefault("EMAIL_USE_TLS", "False"))
    EMAIL_USE_SSL = to_bool(os.environ.setdefault("EMAIL_USE_SSL", "False"))
    DEFAULT_FROM_EMAIL = os.environ.setdefault(
        "DEFAULT_FROM_EMAIL", "noreply@email.com"
    )

    LOGIN_URL = "login"
    PASSWORD_RESET_TIMEOUT = 3600 * 6
    LOGOUT_REDIRECT_URL = "admin"

    GUNICORN_BIND = "127.0.0.1:8000"
    GUNICORN_WORKERS = 2

    INTERNAL_IPS = [
        "127.0.0.1",
    ]
    CORS_ALLOW_ALL_ORIGINS = True
    ALLOWED_HOSTS = ["*"]
    LANGUAGE_CODE = "en-us"
    TIME_ZONE = "UTC"
    USE_I18N = True
    USE_L10N = True
    USE_TZ = True
    STATIC_URL = "/static/"
    STATIC_ROOT = "compiled_static/"
    DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"
    INSTALLED_APPS = [
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.staticfiles",
        "django_q",
        "corsheaders",
        "tuhls.core",
        "rest_framework",
        "djoser",
        "rest_framework.authtoken",
        "drf_spectacular",
    ]

    MIDDLEWARE = [
        "django.middleware.security.SecurityMiddleware",
        "django.contrib.sessions.middleware.SessionMiddleware",
        "django.middleware.locale.LocaleMiddleware",
        "corsheaders.middleware.CorsMiddleware",
        "django.middleware.common.CommonMiddleware",
        "django.middleware.csrf.CsrfViewMiddleware",
        "django.contrib.auth.middleware.AuthenticationMiddleware",
        "django.contrib.messages.middleware.MessageMiddleware",
        "django.middleware.clickjacking.XFrameOptionsMiddleware",
    ]

    TEMPLATES = [
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "DIRS": [],
            "APP_DIRS": True,
            "OPTIONS": {
                "context_processors": [
                    "django.template.context_processors.debug",
                    "django.template.context_processors.request",
                    "django.contrib.auth.context_processors.auth",
                    "django.contrib.messages.context_processors.messages",
                ],
            },
        },
    ]

    DATABASES = {"default": env.db()}
    DATABASES["default"]["ENGINE"] = "django.db.backends.postgresql_psycopg2"

    AUTH_PASSWORD_VALIDATORS = [
        {
            "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
        },
        {
            "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
        },
        {
            "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
        },
        {
            "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
        },
    ]

    LOGGING = {
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            "verbose": {
                "format": "{levelname} {asctime} {module} {process:d} {thread:d}: {message}",
                "style": "{",
            },
            "simple": {
                "format": "{levelname} {message}",
                "style": "{",
            },
        },
        "handlers": {
            "console": {
                "class": "logging.StreamHandler",
                "formatter": "verbose",
            },
        },
        "root": {
            "handlers": ["console"],
            "level": "WARNING",
        },
        "loggers": {
            "django": {
                "handlers": ["console"],
                "level": "INFO",
                "propagate": False,
            },
            "hoarder": {
                "handlers": ["console"],
                "level": "DEBUG",
                "propagate": False,
            },
        },
    }

    CACHES = {"default": env.cache("DEFAULT_CACHE"), "queue": env.cache("QUEUE_CACHE")}

    Q_CLUSTER = {
        "name": "DJRedis",
        "workers": 2,
        "django_redis": "queue",
        "timeout": 10,
        "max_attempts": 3,
        "retry": 120,
        "queue_limit": 10,
        "bulk": 10,
    }
    REST_FRAMEWORK = {
        "DEFAULT_RENDERER_CLASSES": ("rest_framework.renderers.JSONRenderer",),
        "DEFAULT_AUTHENTICATION_CLASSES": [
            "rest_framework_simplejwt.authentication.JWTAuthentication",
            # "rest_framework.authentication.TokenAuthentication",
            "tuhls.core.auth.ExpiringTokenAuthentication",
            "tuhls.core.auth.QueryStringBasedTokenAuthentication",
        ],
        "DEFAULT_PERMISSION_CLASSES": [
            "rest_framework.permissions.IsAuthenticated",
        ],
        "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
        "DEFAULT_THROTTLE_CLASSES": [
            "rest_framework.throttling.AnonRateThrottle",
            "rest_framework.throttling.UserRateThrottle",
        ],
        "TEST_REQUEST_DEFAULT_FORMAT": "json",
    }

    SIMPLE_JWT = {
        "AUTH_HEADER_TYPES": ("JWT",),
    }

    SPECTACULAR_SETTINGS = {
        "TITLE": "API spec",
        "DESCRIPTION": "",
        "CONTACT": {"url": "https://tuhls.com/contact"},
        "LICENSE": {"name": "TLB", "url": "https://tlb.com"},
        "VERSION": "1.7.3",
        "SERVE_INCLUDE_SCHEMA": True,
        "TAGS": [
            {"name": "Settings"},
            {"name": "User"},
            {"name": "User files"},
            {"name": "Groups"},
            {"name": "Group invitations"},
            {"name": "Templates"},
            {"name": "Applications"},
            {"name": "Database tables"},
            {"name": "Database table fields"},
            {"name": "Database table views"},
            {"name": "Database table view filters"},
            {"name": "Database table view sortings"},
            {"name": "Database table grid view"},
            {"name": "Database table rows"},
            {"name": "Database tokens"},
        ],
    }

    @classmethod
    def pre_setup(cls):
        super().pre_setup()
        if not cls.TESTING:
            cls.REST_FRAMEWORK["DEFAULT_THROTTLE_RATES"] = {
                "anon": "60/minute",
                "user": "120/minute",
            }


class BaseDev(BaseCommon):
    INSTALLED_APPS = BaseCommon.INSTALLED_APPS
    MIDDLEWARE = BaseCommon.MIDDLEWARE
    TEMPLATES = BaseCommon.TEMPLATES

    DEBUG = True
    SECRET_KEY = "donotuseinproduction"
    WERKZEUG_DEBUG_PIN = 1234
    TEMPLATE_DEBUG = True
    DEBUG_TOOLBAR = True
    ACCESS_TOKEN_LIFETIME = datetime.timedelta(minutes=60)
    SILENCED_SYSTEM_CHECKS = ["captcha.recaptcha_test_key_error"]

    @classmethod
    def pre_setup(cls):
        super().pre_setup()

        if cls.DEBUG_TOOLBAR:
            from debug_toolbar.settings import PANELS_DEFAULTS

            cls.DEBUG_TOOLBAR_PANELS = PANELS_DEFAULTS + [  # noqa
                "mail_panel.panels.MailToolbarPanel",
            ]
            cls.INSTALLED_APPS.append("debug_toolbar")  # noqa
            cls.INSTALLED_APPS.append("mail_panel")  # noqa

            cls.MIDDLEWARE.append(
                "debug_toolbar.middleware.DebugToolbarMiddleware"
            )  # noqa
            cls.EMAIL_BACKEND = "mail_panel.backend.MailToolbarBackend"
        cls.INSTALLED_APPS.append("django_extensions")  # noqa

        if cls.TEMPLATE_DEBUG:
            cls.TEMPLATES[0]["OPTIONS"]["debug"] = True  # noqa


class BaseProd(BaseCommon):
    DEBUG = False
    SECRET_KEY = values.Value("SECRET_KEY")
    TEMPLATE_DEBUG = False

    EMAIL_BACKEND = "django_q_email.backends.DjangoQBackend"
    DJANGO_REST_PASSWORDRESET_NO_INFORMATION_LEAKAGE = True
    DJANGO_REST_MULTITOKENAUTH_RESET_TOKEN_EXPIRY_TIME = 6

    @classmethod
    def pre_setup(cls):
        super().pre_setup()
        # conn_max_age is ok for single server setups. more than 1 app server on 1 postgres -> try pgbouncer
        # do not set in development -> pg connection leak
        cls.DATABASES["default"]["CONN_MAX_AGE"] = 60

    @classmethod
    def post_setup(cls):
        if env("SENTRY_DSN") != "":
            import sentry_sdk
            from sentry_sdk.integrations.django import DjangoIntegration

            sentry_sdk.init(
                dsn=env("SENTRY_DSN"),
                integrations=[DjangoIntegration()],
                send_default_pii=True,
            )

        cls.TEMPLATES = BaseCommon.TEMPLATES
        cls.TEMPLATES[0]["APP_DIRS"] = False  # noqa
        cls.TEMPLATES[0]["OPTIONS"]["loaders"] = [  # noqa
            (
                "django.template.loaders.cached.Loader",
                [
                    "django.template.loaders.filesystem.Loader",
                    "django.template.loaders.app_directories.Loader",
                ],
            ),
        ]

        cls.MIDDLEWARE.insert(1, "whitenoise.middleware.WhiteNoiseMiddleware")  # noqa
        cls.STATICFILES_STORAGE = (
            "whitenoise.storage.CompressedManifestStaticFilesStorage"
        )



from typing import Any, Tuple

from django.conf import settings
from django.db.models.functions import Now
from django.utils import timezone
from rest_framework.authentication import TokenAuthentication
from rest_framework.exceptions import AuthenticationFailed


class ExpiringTokenAuthentication(TokenAuthentication):
    def authenticate_credentials(self, key: str) -> Tuple[Any, Any]:
        token_user, token = super().authenticate_credentials(key)

        if settings.TOKEN_EXPIRE_SECONDS > 0:
            if (
                timezone.now() - token.created
            ).total_seconds() > settings.TOKEN_EXPIRE_SECONDS:
                token.delete()
                raise AuthenticationFailed("The Token is expired")
            token.created = Now()
            token.save()

        return token_user, token


class QueryStringBasedTokenAuthentication(TokenAuthentication):
    def authenticate(self, request):
        key = request.GET.get("token", None)
        if key:
            return self.authenticate_credentials(key)
        return None



# Generated by Django 3.1.8 on 2021-05-25 21:29

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Invoice',
            fields=[
                ('id', models.BigAutoField(editable=False, primary_key=True, serialize=False)),
                ('gid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('user', models.UUIDField()),
                ('type', models.CharField(blank=True, choices=[('i', 'invoice'), ('e', 'estimate'), ('cn', 'credit note')], max_length=2, null=True)),
                ('number', models.IntegerField(blank=True, null=True)),
                ('name', models.CharField(blank=True, max_length=63, null=True)),
                ('address1', models.CharField(blank=True, max_length=63, null=True)),
                ('address2', models.CharField(blank=True, max_length=63, null=True)),
                ('zip_code', models.CharField(blank=True, max_length=63, null=True)),
                ('city', models.CharField(blank=True, max_length=63, null=True)),
                ('country', models.CharField(blank=True, max_length=63, null=True)),
                ('state', models.CharField(blank=True, max_length=63, null=True)),
                ('company', models.CharField(blank=True, max_length=63, null=True)),
                ('vat', models.CharField(blank=True, max_length=63, null=True)),
                ('payment_reference', models.CharField(blank=True, max_length=63, null=True)),
                ('payment_provider', models.CharField(blank=True, max_length=63, null=True)),
                ('is_paid', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InvoiceDetail',
            fields=[
                ('id', models.BigAutoField(editable=False, primary_key=True, serialize=False)),
                ('gid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=63)),
                ('description', models.CharField(blank=True, max_length=255, null=True)),
                ('amount', models.DecimalField(decimal_places=2, max_digits=8)),
                ('invoice', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='details', to='invoice.invoice')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import os
import sys
from pathlib import PosixPath

from django.apps import AppConfig
from django.utils.autoreload import autoreload_started


def dist_is_editable(dist):
    for path_item in sys.path:
        egg_link = os.path.join(path_item, dist.project_name + ".egg-link")
        if os.path.isfile(egg_link):
            return True
    return False


def my_watchdog(sender, *args, **kwargs):
    import pkg_resources

    for dist in pkg_resources.working_set:
        if dist_is_editable(dist):
            sender.directory_globs[PosixPath(dist.location)] = {"**/*.py"}
            print("additional watch dir", dist.location, dist.project_name)  # noqa


class CoreConfig(AppConfig):
    name = "tuhls.core"

    def ready(self):
        autoreload_started.connect(my_watchdog)



from rest_framework import serializers


class RenameFieldsSerializerMixin:
    def to_internal_value(self, data):
        mapped_data = data.__class__()
        for external, internal in self.field_name_map.items():
            if external in data:
                mapped_data[internal] = data[external]

        return super().to_internal_value(mapped_data)


class RecursiveSerializer(serializers.Serializer):
    def to_representation(self, value):
        serializer = self.parent.parent.__class__(value, context=self.context)
        return serializer.data


class DynamicFieldsHyperlinkedModelSerializer(serializers.HyperlinkedModelSerializer):
    """
    A ModelSerializer that takes an additional `fields` argument that
    controls which fields should be displayed.
    """

    def __init__(self, *args, **kwargs):
        # Don't pass the 'fields' arg up to the superclass
        fields = kwargs.pop("fields", None)

        # Instantiate the superclass normally
        super().__init__(*args, **kwargs)

        if fields is not None:
            # Drop any fields that are not specified in the `fields` argument.
            allowed = set(fields)
            existing = set(self.fields)
            for field_name in existing - allowed:
                self.fields.pop(field_name)



import json

from django.http import QueryDict
from django.test import TestCase


class QueryDictPersistanceTest(TestCase):
    def test_qd(self):
        a_q = QueryDict("a=1&a=2&c=3")
        a_s = a_q.__getstate__()
        a_j = json.dumps(a_s)

        b = QueryDict("", mutable=True)
        b.__setstate__(json.loads(a_j))
        self.assertEqual(b["a"], "2")
        self.assertEqual(b["c"], "3")



from .models import Invoice
from .pdf import create


def invoice_to_pdf(invoice: Invoice):
    return create(
        {
            "to": {
                "name": invoice.name,
                "company": invoice.company,
                "address1": invoice.address1,
                "address2": invoice.address2,
                "zip_code": invoice.zip_code,
                "state": invoice.state,
                "city": invoice.city,
                "country": invoice.country,
            },
            "type": invoice.get_type_name(),
            "creator": "Bilcom GesmbH.",
            "date": invoice.created_at.strftime("%Y-%m-%d %H:%M"),
            "currency_symbol": "$",
            "footer": [
                "Bankverbindung: Erste Bank, IBAN: AT96 2011 1822 6402 1500, BIC GIBAATWWXXX",
                "Bilcom GesmbH. Universitätstraße 4/4, 1090 Wien, office@bilcom.at, +43 1 641 99 85",
            ],
            "items": [
                {
                    "name": x.name,
                    "description": x.description,
                    "amount": x.amount,
                }
                for x in invoice.details.all()
            ],
        }
    )



from django.contrib.auth import get_user_model
from django.contrib.auth.forms import UserChangeForm, UserCreationForm


class CustomUserCreationForm(UserCreationForm):
    class Meta(UserCreationForm):
        model = get_user_model()
        fields = ("email",)


class CustomUserChangeForm(UserChangeForm):
    class Meta:
        model = get_user_model()
        fields = ("email",)



import os
import sys

from dotenv import dotenv_values


def build_env():
    for k, v in dotenv_values(".env").items():
        os.environ[k] = v

    t = os.environ["DJANGO_SETTINGS_MODULE"].split(".")
    os.environ["DJANGO_CONFIGURATION"] = t[-1].capitalize()
    os.environ["DJANGO_SETTINGS_MODULE"] = ".".join(t[:-1])
    print(  # noqa
        os.environ["DJANGO_CONFIGURATION"], os.environ["DJANGO_SETTINGS_MODULE"]
    )
    import configurations

    configurations.setup()


def run():
    from configurations.management import execute_from_command_line

    build_env()
    execute_from_command_line(sys.argv)


def run_example():
    from django.core.management import execute_from_command_line

    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "example.settings")
    execute_from_command_line(sys.argv)


def wsgi():
    from configurations.wsgi import get_wsgi_application

    build_env()
    return get_wsgi_application()



import uuid

from django.contrib.auth.models import (
    AbstractBaseUser,
    BaseUserManager,
    PermissionsMixin,
)
from django.core.mail import send_mail
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _


class CustomModel(models.Model):
    id = models.BigAutoField(primary_key=True, editable=False)
    gid = models.UUIDField(default=uuid.uuid4, unique=True, editable=False)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class EmailUserManager(BaseUserManager):
    def create_user(self, email, password, **extra_fields):
        if not email:
            raise ValueError(_("The Email must be set"))
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save()
        return user

    def create_superuser(self, email, password, **extra_fields):
        extra_fields["is_staff"] = True
        extra_fields["is_superuser"] = True
        extra_fields.setdefault("is_active", True)
        return self.create_user(email, password, **extra_fields)


class EmailUser(AbstractBaseUser, PermissionsMixin, CustomModel):
    email = models.EmailField(_("email address"), unique=True)
    is_staff = models.BooleanField(
        _("staff status"),
        default=False,
        help_text=_("Designates whether the user can log into this admin site."),
    )
    is_active = models.BooleanField(
        _("active"),
        default=True,
        help_text=_(
            "Designates whether this user should be treated as active. "
            "Unselect this instead of deleting accounts."
        ),
    )
    date_joined = models.DateTimeField(_("date joined"), default=timezone.now)
    EMAIL_FIELD = "email"
    USERNAME_FIELD = "email"
    objects = EmailUserManager()

    class Meta:
        verbose_name = _("user")
        verbose_name_plural = _("users")
        abstract = True

    def __str__(self):
        return self.email

    # def clean(self):
    #     super().clean()
    #     self.email = self.__class__.objects.normalize_email(self.email)

    def email_user(self, subject, message, from_email=None, **kwargs):
        send_mail(subject, message, from_email, [self.email], **kwargs)



from django.db import models

from tuhls.core.models import CustomModel


class Request(CustomModel):
    user = models.UUIDField()
    method = models.CharField(max_length=8)
    status = models.IntegerField()
    ip = models.CharField(max_length=16)
    request_data = models.CharField(max_length=8192)
    response_data = models.CharField(max_length=8192)
